//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: 24 January 1998
// Last Modified: Thu Feb 12 18:34:36 GMT-0800 1998
// Filename:      ...sig/examples/sigmidi/improv/synthImprov/switche.cpp
// Syntax:        C++; Synth Improv
//  
// Description: Inverts pitch and attack velocity parameters of
//	keys played on the synthesizer keyboard.  Computer keyboard
//	keys are assigned random attack velocities.
//

#include "synthImprov.h" 


/*----------------- beginning of improvization algorithms ---------------*/

int octave = 4;              // octave range for computer keyboard notes
int keyboardnote = 0;        // computer keyboard note
int lastnote = 0;            // last note which was turned on
MidiMessage noteMessage;     // for reading keyno and velocity (and time)


/*--------------------- maintenance algorithms --------------------------*/

void description(void) {
   cout << endl;
   printboxtop();
   psl(" Switch -- by Craig Stuart Sapp <craig@ccrma.stanford.edu>, 24 Jan 1998");
   psl("");
   psl("  Description: Inverts pitch and attack velocity parameters of");
   psl("    keys played on the synthesizer keyboard.  Computer keyboard");
   psl("    keys are assigned random attack velocities.");
   printintermediateline();
   psl("  User commands:");
   psl("  Computer synthesizer keyboard:");
   psl("      \"0\"-\"9\" = octave number of computer keyboard notes");
   psl("      Notes:           s   d      g    h   j   ");
   psl("                     z   x   c   v   b   n   m  ");
   printboxbottom();
   cout << endl;
} 


void initialization(void) { }


void finishup(void) { }


/*-------------------- main loop algorithms -----------------------------*/

void mainloopalgorithms(void) { 
   while (synth.getNoteCount() > 0) {
      noteMessage = synth.extractNote();
      if (noteMessage.p2() == 0) {
         synth.play(CH_1, lastnote, 0);
      } else {
         synth.play(CH_1, noteMessage.p2(), noteMessage.p1());
         lastnote = noteMessage.p2();
      }
   }
}
      


/*-------------------- triggered algorithms -----------------------------*/

void keyboardchar(int key) { 
   if (isdigit((char)key)) {
      octave = key - '0';   
      return;
   }
   octavekeyboard(key, octave);
}


/*------------------ end improvization algorithms -----------------------*/




