//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu> (from 18Dec1997)
// Creation Date: 4 January 1997
// Last Modified: Wed Feb 11 22:57:46 GMT-0800 1998
// Filename:      ...sig/code/control/improv/improv.cpp
// Syntax:        C++
//
// Description: Basic functions for use in improv environments
//

#include "improv.h"
#include <iostream.h>
#include <iomanip.h>
#include <stdlib.h>


//////////////////////////////
//
// displayInputDevices -- show the user the MIDI input device to 
//	communicate with the specified object.
//

void displayInputDevices(void) {
   MidiInPort tempmidi;
   for (int i=0; i<tempmidi.getNumPorts(); i++) {
      tempmidi.setPort(i);
      cout << "   " << i << ": " << tempmidi.getName() << endl;
   }
}



//////////////////////////////
//
// displayOutputDevices -- show the user the MIDI ouptut device to 
//	communicate with the specified object.
//

void displayOutputDevices(void) {
   MidiOutPort tempmidi;
   for (int i=0; i<tempmidi.getNumPorts(); i++) {
      tempmidi.setPort(i);
      cout << "   " << i << ": " << tempmidi.getName() << endl;
   }
}



//////////////////////////////
//
// printboxtop -- prints a text-graphic line forming the top
//	of a box
//

void printboxtop(void) {
   cout << '\311';
   for (int i=0; i<77; i++) {
      cout << '\315';
   }
   cout << '\273' << endl;
}


void printtopline(void) {
   printboxtop();
}


//////////////////////////////
//
// printboxbottom -- prints a text-graphic line forming the 
//	bottom of a box.
//

void printboxbottom(void) {
   cout << '\310';
   for (int i=0; i<77; i++) {
      cout << '\315';
   }
   cout << '\274' << endl;
}


void printbottomline(void) {
   printboxbottom();
}



//////////////////////////////
//
// printintermediateline -- prints a text-graphic line forming
//	a single horizontal line connecting two sides of a
//	double-lined box.
//

void printintermediateline(void) {
   cout << '\307';
   for (int i=0; i<77; i++) {
      cout << '\304';
   }
   cout << '\266' << endl;
}



//////////////////////////////
//
// printstringline -- prints text inside of an 80-character
//	wide double-lined box.
//

void printstringline(char* aString) {
   int spaceCount = 77 - strlen(aString);
   cout << '\272';
   cout << aString;
   for (int i=0; i<spaceCount; i++) {
      cout << ' ';
   }
   cout << '\272' << endl;
}


void psl(char* aString) {
   printstringline(aString);
}



//////////////////////////////
//
// printcenterline -- centers text inside of an 80-character
//	wide double-lined box.
//

void printcenterline(char* aString) {
   int spaceCount = 77 - strlen(aString);
   int leftCount = spaceCount/2;
   int rightCount = spaceCount - leftCount;
   int i;
   cout << '\272';
   for (i=0; i<leftCount; i++) {
      cout << ' ';
   }
   cout << aString;
   for (i=0; i<rightCount; i++) {
      cout << ' ';
   }
   cout << '\272';
   cout << endl;
}


void pcl(char* aString) {
   printcenterline(aString);
}



//////////////////////////////
//
// midiscale -- converts a value in the range from 0 to 127
//      into a number in a new range.  For example the value
//      64 scaled to the range from 0 to 2 would be 1.
//

int midiscale(int value, int min, int max) {
   return value >= 127 ? max : (int)(value/127.0*(max-min+1)+min);
}




